
## How to add new Events

### Steps to add new Event
* Add entry to [`EventType`](./common/v1/event_code.proto) enum. Please follow these rules:
    * Add proper namespace to your event code, e.g. `ADS_XXX`, e.g. `ADS_VIDEO_AUDIO_ROLL_REQUESTED` . Please find available namespace prefixes in `conventions` section.
    * Assign first not occupied index to new enum entry
    * keep the 'past form' of the code, e.g. don't use `VIDEO_AUDIO_ROLL_REQUEST`, but `VIDEO_AUDIO_ROLL_REQUESTED` instead
    
    Example:
    ```
        enum EventCode {
          EVENT_CODE_UNSPECIFIED = 0;
        ...
    
        // Your events codes
          ADS_NEW_EVENT = 13;
    }
    ```

* Create `Event` protobuf file:
    * add a file under [`tunein/clarity/ueapi/events/v1/`](./events/v1/)
    * for file name use lowercased event code, e.g. `ads_video_audio_roll_requested.proto`
    * add package descriptor: `package tunein.clarity.ueapi.events.v1;`
    * message name should correspond to event code, but should be camel cased. Name should follow `Context => Object => Action` convention e.g. `AdsPrerollRequestedEvent`. 
    * add common attributes to your event and corresponding imports:
    ```
        import "google/protobuf/timestamp.proto";
        import "tunein/clarity/ueapi/common/v1/context.proto";
        import "tunein/clarity/ueapi/common/v1/event_code.proto";
        import "tunein/clarity/ueapi/common/v1/event_type.proto";
        
        message YourNewEventEvent {
              string device_id = 1; // deviceId
              string message_id = 2; // auto generated UUID
              google.protobuf.Timestamp event_ts = 3;
              tunein.clarity.ueapi.common.v1.EventType type = 6; // EventType.EVENT_TYPE_TRACK -> change to proper type
              tunein.clarity.ueapi.common.v1.EventCode event = 7; // EventCode.ADS_XXXX -> change to your new event code
              tunein.clarity.ueapi.common.v1.Context context = 8;
            
        
        }
    ```
    * add event specific attributes. Start with index 20 as in the example : [AdsPlaybackStartedEvent](./events/v1/ads_playback_started.proto)
    * if new event indicates any failure on client side add common 'failure description attributes'. Example: [`AdsPlaybackFailedEvent`](./events/v1/ads_playback_failed.proto)
    * if relevant, add schema documentation / comments to new event protto file
    
* Add attributes to common Event schema -> [event.proto](./v1/event.proto)
    * if any of the attributes in the new event is not present in [event.proto](./v1/event.proto). if not, please add them there with incremented indices
    
* Add / update documentation (topology / samples etc)
    * create doc file if not exist - file name convention `topology_xxxx.md'. Example: [topology_video_audio_roll_events.md](topology_video_audio_roll_events.md)
        * add link to your events topology documentation in main [Readme file](../../../../README.md)
    * if doc file already exist, document new events. Example: [topology_video_audio_roll_events.md](topology_video_audio_roll_events.md)



### conventions

* Event code namespaces
    You should use one of the following namespaces if your event corresponds to the domain, or add new if needed:
    * `ADS_`
    * `USER_`
    * `LISTEN_`

### Resources
1. [Protobuf Style Guide](https://protobuf.dev/programming-guides/style/)
2. [Protobuf Schema Documentation](https://protobuf.dev/programming-guides/proto3/)
